/*
;Funktion:                      Except4.cpp
;                               Demonstration eines Ausnahmefehlers mit erweiterter
;                               komplexer Ausnahmefehler-Behandlung mit separater 
,                               Behandlungsroutine im Anwenderprogramm

;getestet unter:                Konsole Windows NT/2000/XP/7/10
;funktioniert nicht unter:      -

;uebersetzen mit:               cl Except4.cpp      als 32bit-Programm

;Aufgabenstellung:              siehe Vorlesung Rechnertechnik 2, Abschnitt E.4

;Copyright:                     (C) 2004-2021  Werner Zimmermann

;############################################################################
*/
#define _CRT_SECURE_NO_DEPRECATE	//Unterdrckt Sicherheitswarnung fr scanf()
#ifdef  _WIN64	
    #define isVer "WIN64"	
#else
#ifdef  _WIN32
    #define isVer "WIN32"	
#endif
#endif

#include <windows.h>
#include <stdio.h>
#include <conio.h>

int ergebnis, dividend=1, divisor;

    
//Routine zur Behandlung eines Ausnahmefehlers
DWORD exHandler(EXCEPTION_POINTERS *pEx)
{   
    printf("Ausnahmefehlerbehandlung mit erweiterten Fehlerinformationen:\n");
    printf("Exception Code            =  %8Xh (EXCEPTION_INT_DIVIDE_BY_ZERO)\n", pEx->ExceptionRecord->ExceptionCode);
    printf("Exception Address         =  %8Xh\n", (unsigned int) pEx->ExceptionRecord->ExceptionAddress);
#ifdef _WIN64
    printf("Context Record:       RIP =  %8llXh     RAX = %llXh  ... (CPU-Register)\n", pEx->ContextRecord->Rip, pEx->ContextRecord->Rax);
#else
    printf("Context Record:       EIP =  %8lXh      EAX = %lXh   ... (CPU-Register)\n", pEx->ContextRecord->Eip, pEx->ContextRecord->Eax);
#endif
    
    if (pEx->ExceptionRecord->ExceptionCode != EXCEPTION_INT_DIVIDE_BY_ZERO) //Nur Divisionsfehler werden behandelt
        return EXCEPTION_CONTINUE_SEARCH;   //...bei anderen Fehlern wird eine uebergeordnete Behandlungsroutine gesucht.
    
    printf("Fehler! Wollen Sie einen neuen Divisor eingeben: J/N ?    ");
    if (toupper(_getch()) == 'J')
    {   printf("\nDivisor  =    ");
        scanf("%d", &divisor);
        return EXCEPTION_CONTINUE_EXECUTION;        //Fehler korrigiert, nochmal dividieren
    } else
        return EXCEPTION_EXECUTE_HANDLER;           //Fehler wurde nicht korrigiert, __try-Block beenden
}


void main(void)
{   printf("(C) 2002-2021 -- Zimmermann, HS Esslingen -- %s --\n\n", isVer); 

    printf("Divisionsprogramm: Geben Sie einen Dividenden und einen Divisor ein!\n\n");
    printf("Wenn Sie Divisor=0 eingeben, wird ein Ausnahmefehler ausgeloest und\n");
    printf("die Ausnahmefehler-Behandlung des Anwenderprogramms aufgerufen. Diese\n");
    printf("gibt eine Fehlermeldung aus und erlaubt Ihnen, den Divisor zu korrigieren.\n\n");
    printf("Sie koennen das Programm 'normal' beenden, indem Sie Dividend=0 eingeben\n\n");
    
    while (1)
    {   printf("\nDividend =    ");
        scanf("%d", &dividend);
        if (dividend == 0)
            break;
        printf("Divisor  =    ");
        scanf("%d", &divisor);

        __try
        {   
            ergebnis = dividend / divisor;      //<-- Befehl loest bei Divisor=0 Ausnahmefehler aus
            printf("Quotient =    %d / %d = %d\n", dividend, divisor, ergebnis);
        
        } __except( exHandler( GetExceptionInformation() ) ) 
        {   printf("\nUngueltiger Divisor - Fehler nicht korrigiert\n");
        }
     }
}
