/*
;Funktion:                      ShowPageInfo
;                               Beispielprogramm zum Anzeigen Page Informationen

;getestet unter:                Konsole Windows NT/2000/XP/7/10
;funktioniert nicht unter:      -

;uebersetzen mit:               cl -Zi ShowPageInfo.cpp         als 32bit-Programm

;Aufgabenstellung:              siehe Vorlesung Rechnertechnik 2, Abschnitt E.5

;Copyright:                     (C) 2002-2021  Werner Zimmermann

;############################################################################
*/
#ifdef  _WIN64	
    #define isVer "WIN64"	
#else
#ifdef  _WIN32
    #define isVer "WIN32"	
#endif
#endif

#include <windows.h>
#include <stdio.h>
#include <conio.h>

DWORD myVariable;                                               //Globale Variable


/* Anzeige der Page-Groesse und des virtuellen Adressbereichs fuer Anwenderprogramme 
*/
void pagingBasicInfo(void)
{   SYSTEM_INFO systemInfo;                                     //Windows-Struktur zur Aufnahme der Informationen
    
    GetSystemInfo(&systemInfo);                                 //Windows-Funktion zur Abfrage der Infos
    printf("Groesse einer Page in Byte                        = %d\n",        systemInfo.dwPageSize);
    printf("Virtueller Adressbereich fuer Anwendungsprogramme = %p ... %ph\n", systemInfo.lpMinimumApplicationAddress,
                                                                systemInfo.lpMaximumApplicationAddress);
}

/* Anzeige von Informationen ueber die Page, die die Adresse 'address' enthaelt
*/
void showPageInfo(void *address)
{   MEMORY_BASIC_INFORMATION memoryInfo;                        //Windows-Struktur zur Aufnahme der Informationen
    DWORD i;
    
    i = VirtualQuery(address, &memoryInfo, sizeof(MEMORY_BASIC_INFORMATION));   //Windows-Funktion zur Abfrage der Infos
    if (i == 0)                                                 //Keine Information zu Pages im Kernel-Bereich
    {   printf("--- Virtuelle Adresse                          %ph\n", address);
        printf("--- Attribute der Seite:                      ");
        printf(" U = 0 kernel mode only\n");
        return;
    }
    
    printf("--- Virtuelle Adresse                          %ph\n", address);
    printf("--- Virtual Baseaddress  der zugehoerigen Page %ph\n", memoryInfo.BaseAddress);
    printf("--- Attribute der Seite:                      ");
    switch (memoryInfo.Protect)
    {   case PAGE_NOACCESS:
                printf(" P = 0  page not present\n");
                break;
        case PAGE_EXECUTE_READ:
        case PAGE_READONLY: 
                printf(" P = 1  U = 1  W = 0  read only\n");
                break;
        case PAGE_READWRITE:
                printf(" P = 1  U = 1  W = 1  read/write\n");
                break;
        default:
                printf(" Unbekannte Attribute %Xh\n", memoryInfo.Protect);
    }
}

/* Hauptprogramm
*/
void main(void)
{   DWORD i;

    printf("(C) 2002-2021 -- Zimmermann, HS Esslingen -- %s --\n\n", isVer); 

    printf("\n********** ShowPageInfo *************************************************\n");
    pagingBasicInfo();
    
    printf("\nWeiter mit Tastendruck\n\n");
    while(!_kbhit());_getch();

    printf("*** Virtuelle Adresse 0h unterhalb des Bereichs fuer Anwendungsprogramme\n");
    showPageInfo((void*) 0x00000000);

    printf("\nWeiter mit Tastendruck\n\n");
    while(!_kbhit());_getch();

    printf("*** Virtuelle Adresse 8000 0000h im Betriebsystembereich\n");
    showPageInfo((void*) 0x80000000);

    printf("\nWeiter mit Tastendruck\n\n");
    while(!_kbhit());_getch();

    printf("*** C-Funktion 'main()' im Codesegment des Programms\n");
    showPageInfo(&main); 
    printf("\nWeiter mit Tastendruck\n\n");
    while(!_kbhit());_getch();

    printf("*** Globale Variable 'myVariable' im Datensegment des Programms\n");
    showPageInfo(&myVariable);

    printf("\nWeiter mit Tastendruck\n\n");
    while(!_kbhit());_getch();

    printf("*** Lokale Variable 'i' im Stacksegment des Programms\n");
    showPageInfo(&i);
}